
const { resolve } = require('path');

const { wrap } = require('co');

const pkg = require('../../package.json');
const pm2p = require('../..');

function start() {
  return pm2p.start(resolve(__dirname, 'server.js'));
}

function del() {
  return pm2p.delete('server');
}

describe('main export', function () {

  beforeEach('delete server', function () {
    return del().catch(() => {});
  });

  after('killDaemon', function () {
    return pm2p.killDaemon();
  });

  it('start returns description', wrap(function* () {
    let ret;
    ret = yield start();
    ret[0].pm2_env.unstable_restarts.should.eql(0);
  }));

  it('start and stop', wrap(function* () {
    yield start();
    yield pm2p.stop('server');
  }));

  it('sendSignalToProcessName', wrap(function* () {
    yield start();
    yield pm2p.sendSignalToProcessName('SIGINT', 'server');
    const ret = yield pm2p.describe('server');
    ret[0].pm2_env.unstable_restarts.should.eql(1);
    ret[0].monit.memory.should.be.above(0);
  }));

  it('list', wrap(function* () {
    yield start();
    const ret = yield pm2p.list();
    ret.length.should.equal(1);
    ret[0].monit.memory.should.be.above(0);
  }));

  it('restart', wrap(function* () {
    yield start();
    const ret = yield pm2p.restart('server');
    ret[0].status.should.equal('online');
  }));

  it('gracefulReload', wrap(function* () {
    yield start();
    const ret = yield pm2p.gracefulReload('server');
    ret[0].status.should.eql('online');
  }));

  it('getProcessIdByName', wrap(function* () {
    yield start();
    const ret = yield pm2p.getProcessIdByName('server');
    ret.should.deep.equal([0]);
  }));

  it('dump and resurrect', wrap(function* () {
    yield start();
    const ret = yield pm2p.dump();
    ret.success.should.eql(true);
    yield pm2p.killDaemon();
    yield pm2p.resurrect();
    const descriptions = yield pm2p.describe('server');
    descriptions.length.should.equal(1);
  }));

  it('flush', wrap(function* () {
    const ret = yield pm2p.flush();
    ret.should.deep.equal([]);
  }));

  it('rejects if something is wrong', wrap(function* () {
    try {
      yield pm2p.delete('server');
      throw 'foo';
    } catch (ex) {
      ex.message.should.match(/not found/);
    }
  }));

  it('pingDaemon', wrap(function* () {
    const ret = yield pm2p.pingDaemon();
    ret.msg.should.equal('pong');
  }));

  it('reloadLogs', wrap(function* () {
    const ret = yield pm2p.reloadLogs();
    ret.should.deep.equal({});
  }));

  it('getVersion', wrap(function* () {
    const ret = yield pm2p.getVersion();
    ret.should.equal(pkg.dependencies.pm2);
  }));

});
