'use strict';

const Pm2 = require('pm2');

const { promisify } = require('@carnesen/util');

const log = require('./log');

const opts = {};
if (process.env.NODE_ENV === 'test') {
  opts.pm2_home = '.pm2';
}

const pm2 = new Pm2.custom(opts);

const connect = promisify(pm2.connect.bind(pm2));
const disconnect = promisify(pm2.connect.bind(pm2));

function makeGeneratorFunction(name) {

  const promisified = promisify(pm2[name].bind(pm2));

  return function* (...args) {

    log.debug(`Calling pm2.${ name } with args ${ args }`);

    let result;
    let connected = false;

    try {

      yield connect();
      log.debug('Connected to pm2 God daemon');
      connected = true;
      result = yield promisified(...args);

    } catch (ex) {

      log.debug(ex);
      throw ex;

    } finally {

      if (connected) {
        try {
          yield disconnect();
        } catch (ex) {
          log.warn(ex);
        }
      }

    }

    return result;

  };

}

makeGeneratorFunction.pm2 = pm2;

module.exports = makeGeneratorFunction;
